<?php
namespace MyMeals\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use MyMeals\Services\JWT;
use MyMeals\Services\Meals as MealsService;
use MyMeals\Services\Shifts;
use MyMeals\Services\DeliveryPoints;

class Meals {

    protected $mealsService;
    protected $shiftsService;
    protected $deliveryPointsService;
    
    const APPROVED = "APROVADA";

    public function __construct(MealsService $mealsService, Shifts $shiftsService, DeliveryPoints $deliveryPointsService) {
        $this->mealsService = $mealsService;
        $this->shiftsService = $shiftsService;
        $this->deliveryPointsService = $deliveryPointsService;
    }

    public function getMeals(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $filterCriteria = $request->getFilterCriteria();
            $dataSet = $this->mealsService->getMeals($filterCriteria);
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getHistory(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $filterCriteria = $request->getFilterCriteria();
            $dataSet = $this->mealsService->getHistory($filterCriteria);
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getAllHistory(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $filterCriteria = $request->getFilterCriteria();
            $dataSet = $this->mealsService->getAllHistory($filterCriteria);
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function saveMeal(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $meal = $row["meal"];
            $shifts = $row["shifts"];
            $emailMessages = $row["emailMessages"];
            $deliveryPoints = $row["deliveryPoints"];
            list($originalQuantity, $logCode) = $this->mealsService->getOriginalQuantityAndLogCode($meal);
            $meal["QTCOMEORIGINAL"] = $originalQuantity;
            $this->mealsService->updateMeal($meal, $originalQuantity, $emailMessages);
            $nrhistoricomymeals = $this->mealsService->saveHistory($meal);
            $meal['NRHISTORICOMYMEALS'] = $nrhistoricomymeals;
            $this->mealsService->createLog($logCode, $meal);
            $this->shiftsService->saveShifts($meal, $shifts, $logCode);
            $this->deliveryPointsService->saveDeliveryPoints($meal, $deliveryPoints, $logCode);
            $response->addDataSet(new DataSet("meal", array($meal)));
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getDeliveryPointsAndShiftsHistory(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $nrhistoricomymeals = $row['NRHISTORICOMYMEALS'];
            $qtcomensaisHistory = $this->mealsService->getQtcomensaisHistory($nrhistoricomymeals);
            $shiftsHistory = $this->mealsService->getShiftsHistory($nrhistoricomymeals);
            $deliveryPointsHistory = $this->mealsService->getDeliveryPointsHistory($nrhistoricomymeals);
            $dados = array (
               "COMENSAIS" => $qtcomensaisHistory,
               "TURNOS" => $shiftsHistory,
               "PONTO_ENTREGA" => $deliveryPointsHistory
            );
            $response->addDataSet(new DataSet("dadosRetorno", $dados));
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
}