<?php
namespace MyMeals\Services;

use Zeedhi\Framework\DataSource\FilterCriteria;
use MyMeals\Util\ParameterBag;
use MyMeals\Util\DateUtil;
use MyMeals\Util\RequestHeader;

class Meals {

    const APPROVED = "APROVADA";
    const PENDING = "PENDENTE";
    const CHANGE_TYPES = array(
        "ACIMA"=> "acima do limite parametrizado",
        "ABAIXO"=> "abaixo do limite parametrizado",
        "TODAS"=> ""
    );
    
    protected $entityManager;
    protected $dataSourceManager;
    protected $parameterBag;
    protected $connection;
    protected $smtpService;

    public function __construct($entityManager, $dataSourceManager, ParameterBag $parameterBag, $connection, $smtpService) {
        $this->entityManager = $entityManager;
        $this->dataSourceManager = $dataSourceManager;
        $this->parameterBag = $parameterBag;
        $this->connection = $connection;
        $this->smtpService = $smtpService;
        $this->templatesDir = __DIR__.DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'Templates'.DIRECTORY_SEPARATOR;
    }

    public function getMeals(FilterCriteria $filterCriteria) {
        $filterCriteria->setDataSourceName("meals");
        $newFilterCriteria = $this->parameterBag->getPopulatedFilterFromSession($filterCriteria);
        return $this->dataSourceManager->findBy($newFilterCriteria);
    }
    
    public function getHistory(FilterCriteria $filterCriteria) {
        $filterCriteria->setDataSourceName("historico");
        $newFilterCriteria = $this->parameterBag->getPopulatedFilterFromSession($filterCriteria);
        return $this->dataSourceManager->findBy($newFilterCriteria);
    }
    
    public function getAllHistory(FilterCriteria $filterCriteria) {
        $filterCriteria->setDataSourceName("historicoTotal");
        $newFilterCriteria = $this->parameterBag->getPopulatedFilterFromSession($filterCriteria);
        return $this->dataSourceManager->findBy($newFilterCriteria);
    }

    public function getOriginalQuantityAndLogCode($meal) {
        $filterCriteria = new FilterCriteria("originalQuantity", $this->getSessionFilter());
        $filterCriteria->addCondition("DTCARDAPIO", DateUtil::truncDate($meal["DTCARDAPIO"]));
        $filterCriteria->addCondition("CDSERVFILI", $meal["CDSERVFILI"]);
        $newFilterCriteria = $this->parameterBag->getPopulatedFilterFromSession($filterCriteria);
        $rows = $this->dataSourceManager->findBy($newFilterCriteria)->getRows();
        $originalQuantity = $meal["QTANTERIOR"];
        $logCode = 1;
        if (count($rows) > 0) {
            if (isset($rows[0]["QTCOMEORIGINAL"])) {
                $originalQuantity = $rows[0]["QTCOMEORIGINAL"];
            }

            if (isset($rows[0]["LOG"])) {
                $logCode = intval($rows[0]["LOG"]) + 1;
            }
        }
        return array(intval($originalQuantity), $logCode);
    }

    public function getParamServicoOper($meal) {
        $filterCriteria = new FilterCriteria("paramservicooper", $this->getSessionFilter());
        $filterCriteria->addCondition("CDSERVFILI", $meal["CDSERVFILI"]);
        $newFilterCriteria = $this->parameterBag->getPopulatedFilterFromSession($filterCriteria);
        return $this->dataSourceManager->findBy($newFilterCriteria)->getRows()[0];
    }

    private function getSessionFilter() {
        return array(
            array(
                "columnName" => "CDFILIAL",
                "operator" => "=",
                "value" => "SESSION",
            ),
            array(
                "columnName" => "NRORG",
                "operator" => "=",
                "value" => "SESSION"
            )
        );
    }
    private function getSessionValue($name) {
        $token = RequestHeader::getKey("session");
        $session = JWT::getTokenData($token);
        return isset($session[$name]) ? $session[$name] : NULL;
    }

    public function updateMeal(&$meal, $originalQuantity, $emailMessages) {
        $quantity = intval($meal["COMENSAIS"]);
        
        $paramServicoOper = $this->getParamServicoOper($meal);

        $additionPercent = floatval($meal["VRACRESCIMO"]) / 100;
        $reductionPercent = floatval($meal["VRREDUCAO"]) / 100;
        $quantityAbove = $originalQuantity + $originalQuantity * $additionPercent;
        $quantityBelow = $originalQuantity - $originalQuantity * $reductionPercent;

        if ($quantity > $quantityAbove) {
            $meal["IDQTDEAPROVADA"] = self::PENDING;
            if ($paramServicoOper["IDNOTIFICAREFE"] == "NOTIFICA_ACIMA_LIMITE" || $paramServicoOper["IDNOTIFICAREFE"] == "NOTIFICA_TODAS") {
                $emailMessages["emailDescription"] = str_replace("__TYPE__", $emailMessages["emailAcima"], $emailMessages["emailDescription"]);
                $this->sendMail($meal, $paramServicoOper, "ACIMA", $emailMessages);
            }
        } else if ($quantity < $quantityBelow) {
            $meal["IDQTDEAPROVADA"] = self::PENDING;
            if ($paramServicoOper["IDNOTIFICAREFE"] == "NOTIFICA_ABAIXO_LIMITE" || $paramServicoOper["IDNOTIFICAREFE"] == "NOTIFICA_TODAS") {
                $emailMessages["emailDescription"] = str_replace("__TYPE__", $emailMessages["emailAbaixo"], $emailMessages["emailDescription"]);
                $this->sendMail($meal, $paramServicoOper, "ABAIXO", $emailMessages);
            }
        } else {
            if ($paramServicoOper["IDFORMALTERA"] == "APROVACAO" || $meal["COMENSAIS"] == 0) {
                $meal["IDQTDEAPROVADA"] = self::PENDING;
            } else {
                $meal["IDQTDEAPROVADA"] = self::APPROVED;
            }
            if ($paramServicoOper["IDNOTIFICAREFE"] == "NOTIFICA_TODAS") {
                $emailMessages["emailDescription"] = str_replace("__TYPE__", "", $emailMessages["emailDescription"]);
                $this->sendMail($meal, $paramServicoOper, "TODAS", $emailMessages);
            }
        }
        $meal["TOTAL"] = intval($meal["TOTAL"]) + ($quantity - intval($meal["QTANTERIOR"]));
        if ($meal["IDQTDEAPROVADA"] === self::APPROVED) {
            $update = "UPDATE CARDADIA SET QTCOMESTICAR = :QTCOMESTICAR, QTCLIESTICAR = :QTCLIESTICAR WHERE NRCARDAPIO = :NRCARDAPIO AND CDSERVFILI = :CDSERVFILI AND CDFILIAL = :CDFILIAL AND NRORG = :NRORG";
            $params = array(
                "QTCOMESTICAR" => $meal["TOTAL"],
                "QTCLIESTICAR" => $meal["COMENSAIS"],
                "NRCARDAPIO" => $meal["NRCARDAPIO"],
                "CDSERVFILI" => $meal["CDSERVFILI"],
                "CDFILIAL" => $meal["CDFILIAL"],
                "NRORG" => $meal["NRORG"]
            );
            $sth = $this->connection->prepare($update);
            $sth->execute($params);
        }
    }

    public function sendMail($meal, $paramServicoOper, $type, $emailMessages) {
        $token = RequestHeader::getKey("session");
        $session = JWT::getTokenData($token);
        $emails = array($paramServicoOper["DSEMAILNOTIF"], $session['dsemailoper']);
        $mailBody = $this->buildMail($meal, $paramServicoOper, $type, $emailMessages);
        $this->smtpService->setFromName($emailMessages["emailFromName"]);
        $this->smtpService->setSubject($emailMessages["emailSubject"]);
        $this->smtpService->sendMail($mailBody, $emails);
    }

    public function fileGetContentUtf8($fileName)
    {
        $content = file_get_contents($fileName);
        return mb_convert_encoding($content, 'UTF-8', mb_detect_encoding($content, 'UTF-8, ISO-8859-1', true));
    }

    public function buildMail ($meal, $paramServicoOper, $type, $emailMessages) {
        $title = $emailMessages["emailDescription"];
        $content = $this->fileGetContentUtf8($this->templatesDir."mail.html");
        $dtcardapio = substr($meal["DTCARDAPIO"], 0, 10);
        $nmoperador = $this->getSessionValue("nmoperador");
        $nmfilial = $this->getSessionValue("nmfilial");
        $footerMessage = $emailMessages["emailFooter"];
        $status = $meal["IDQTDEAPROVADA"] == self::PENDING ? $emailMessages["emailPendente"] : $emailMessages["emailAprovada"];

        $search = array(
            "%title%",
            "%dateLabel%",
            "%dateValue%",
            "%operatorLabel%",
            "%operatorValue%",
            "%unitLabel%",
            "%unitValue%",
            "%serviceLabel%",
            "%serviceValue%",
            "%originalAmountLabel%",
            "%originalAmountValue%",
            "%changedAmountLabel%",
            "%changedAmountValue%",
            "%statusLabel%",
            "%statusValue%",
            "%footerMessage%"
        );
        $replace = array(
            $title,
            $emailMessages["emailDataCardapio"],
            $dtcardapio,
            $emailMessages["emailOperador"],
            $nmoperador,
            $emailMessages["emailUnidade"],
            $nmfilial,
            $emailMessages["emailServico"],
            $meal["NMSERVICO"],
            $emailMessages["emailQtOri"],
            $meal["QTCOMEORIGINAL"],
            $emailMessages["emailQtAlt"],
            $meal["COMENSAIS"],
            $emailMessages["emailSituacao"],
            $status,
            $footerMessage
        );
        return str_replace($search, $replace, $content);
    }

    public function createLog($logCode, $meal) {
        $dsparametros = "NOTIFICA: " . $meal["IDNOTIFICAREFE"] .
                    ' - ' . "HORA ALTERACAO: " . $meal["HRALTERACLI"] .
                    ' - ' . "VALOR ACRESCIMO: " . $meal["VRACRESCIMO"] .
                    ' - ' . "VALOR REDUCAO: " . $meal["VRREDUCAO"] .
                    ' - ' . "NÚMERO DE DIAS ALTERADO: " . $meal["NRDIASALT"];
        $log = array(
            "NRCONFIRMAREFE"=> $logCode,
            "CDFILIAL"=> $meal["CDFILIAL"],
            "CDSERVFILI"=> $meal["CDSERVFILI"],
            "NRCARDAPIO"=> $meal["NRCARDAPIO"],
            "DTCARDAPIO"=> DateUtil::truncDate($meal["DTCARDAPIO"]),
            "CDCLIENTE"=> null,
            "CDDESTFILI"=> null,
            "CDTURNO"=> null,
            "QTCOMEORIGINAL"=> $meal["QTCOMEORIGINAL"],
            "QTCOMEALTERADA"=> $meal["COMENSAIS"],
            "QTTOTALALTERADA"=> $meal["TOTAL"],
            "IDQTDEAPROVADA"=> $meal["IDQTDEAPROVADA"],
            "DSPARAMETROS"=> $dsparametros,
            "NRORG"=> $meal["NRORG"],
            "IDATIVO"=> "S",
            "DTINCLUSAO"=> DateUtil::currentDateTimeString(),
            "DTULTATU"=> DateUtil::currentDateTimeString(),
            "NRORGINCLUSAO"=> $meal["NRORG"],
            "CDOPERINCLUSAO"=> $meal["CDOPERADOR"],
            "NRORGULTATU"=> $meal["NRORG"],
            "CDOPERULTATU"=> $meal["CDOPERADOR"],
            "CDPRATO"=> null,
            "NRHISTORICOMYMEALS"=> $meal['NRHISTORICOMYMEALS']
        );

        $insert = "INSERT INTO CONFIRMAREFE (NRCONFIRMAREFE, CDFILIAL, CDSERVFILI, NRCARDAPIO, DTCARDAPIO, CDPRATO, CDCLIENTE, CDDESTFILI, CDTURNO, QTCOMEORIGINAL, QTCOMEALTERADA, QTTOTALALTERADA, IDQTDEAPROVADA, DSPARAMETROS, NRORG, IDATIVO, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU, NRHISTORICOMYMEALS) VALUES (:NRCONFIRMAREFE, :CDFILIAL, :CDSERVFILI, :NRCARDAPIO, :DTCARDAPIO, :CDPRATO, :CDCLIENTE, :CDDESTFILI, :CDTURNO, :QTCOMEORIGINAL, :QTCOMEALTERADA, :QTTOTALALTERADA, :IDQTDEAPROVADA, :DSPARAMETROS, :NRORG, :IDATIVO, :DTINCLUSAO, :DTULTATU, :NRORGINCLUSAO, :CDOPERINCLUSAO, :NRORGULTATU, :CDOPERULTATU, :NRHISTORICOMYMEALS)";
        $sth = $this->connection->prepare($insert);
        $sth->execute($log);
    }
    
    public function getSeqHistoricoMyMeals(){
        $query = "SELECT SEQ_HISTORICOMYMEALS.NEXTVAL AS SEQ FROM DUAL";
        $params = array();
        return $this->entityManager->getConnection()->fetchAll($query, $params);
    }
    
    public function saveHistory($dadosMeal) {
        $nrhistoricomymeals = $this->getSeqHistoricoMyMeals()[0]['SEQ'];
        $dadosInsert = array(
            "NRHISTORICOMYMEALS"=> $nrhistoricomymeals,
            "CDOPERADOR"=> $dadosMeal["CDOPERADOR"],
            "CDFILIAL"=> $dadosMeal["CDFILIAL"],
            "CDSERVFILI"=> $dadosMeal["CDSERVFILI"],
            "DTCARDAPIO" => $dadosMeal["DTCARDAPIO"],
            "QTCOMENSAIS"=> $dadosMeal["COMENSAIS"],
            "IDSITUACAO"=> $dadosMeal["IDQTDEAPROVADA"] == 'APROVADA' ? 'APROVADO' : $dadosMeal["IDQTDEAPROVADA"],
            "DSOBSERVACAO"=> isset($dadosMeal["OBSERVACAO"]) ? $dadosMeal["OBSERVACAO"] : "",
            "DTINCLUSAO"=> DateUtil::currentDateTimeString(),
            "DTULTATU"=> DateUtil::currentDateTimeString(),
            "NRORGINCLUSAO"=> $dadosMeal["NRORG"],
            "CDOPERINCLUSAO"=> $dadosMeal["CDOPERADOR"],
            "NRORGULTATU"=> $dadosMeal["NRORG"],
            "CDOPERULTATU"=> $dadosMeal["CDOPERADOR"]
        );
        $insert = "INSERT INTO HISTORICOMYMEALS  (NRHISTORICOMYMEALS, CDOPERADOR, CDFILIAL, CDSERVFILI, DTCARDAPIO, DTHISTORICO, QTCOMENSAIS, IDSITUACAO, DSOBSERVACAO, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU) 
                VALUES (:NRHISTORICOMYMEALS, :CDOPERADOR, :CDFILIAL, :CDSERVFILI, :DTCARDAPIO, TRUNC(SYSDATE), :QTCOMENSAIS, :IDSITUACAO, :DSOBSERVACAO, :DTINCLUSAO, :DTULTATU, :NRORGINCLUSAO, :CDOPERINCLUSAO, :NRORGULTATU, :CDOPERULTATU)";
        $sth = $this->connection->prepare($insert);
        $sth->execute($dadosInsert);
        
        return $nrhistoricomymeals;
    }
    
    public function getQtcomensaisHistory($nrhistoricomymeals){
        $query = "SELECT * FROM CONFIRMAREFE WHERE CDTURNO IS NULL AND CDCLIENTE IS NULL AND NRHISTORICOMYMEALS = :NRHISTORICOMYMEALS";
        $params = array(
            "NRHISTORICOMYMEALS" => $nrhistoricomymeals
        );
        return $this->entityManager->getConnection()->fetchAll($query, $params);
    }
    
    public function getShiftsHistory($nrhistoricomymeals){
        $query = "SELECT C.*, T.NMTURNO
                FROM CONFIRMAREFE C
                LEFT JOIN TURNOSERV T ON C.CDFILIAL = T.CDFILIAL
                                    AND C.CDSERVFILI = T.CDSERVFILI
                                    AND C.CDTURNO = T.CDTURNO
                WHERE C.CDTURNO IS NOT NULL
                AND NRHISTORICOMYMEALS = :NRHISTORICOMYMEALS";
        $params = array(
            "NRHISTORICOMYMEALS" => $nrhistoricomymeals
        );
        return $this->entityManager->getConnection()->fetchAll($query, $params);
    }
    
    public function getDeliveryPointsHistory($nrhistoricomymeals){
        $query = "SELECT C.*, DF.NMDESTFILI
                FROM CONFIRMAREFE C
                LEFT JOIN CARDACLI  CC ON CC.CDFILIAL = C.CDFILIAL
                                        AND CC.NRCARDAPIO = C.NRCARDAPIO
                                        AND C.CDDESTFILI = CC.CDDESTFILI
                LEFT JOIN DESTFILI  DF ON C.CDFILIAL = DF.CDFILIAL
                                AND C.CDDESTFILI = DF.CDDESTFILI
                WHERE C.CDCLIENTE IS NOT NULL
                AND NRHISTORICOMYMEALS = :NRHISTORICOMYMEALS";
        $params = array(
            "NRHISTORICOMYMEALS" => $nrhistoricomymeals
        );
        return $this->entityManager->getConnection()->fetchAll($query, $params);
    }
}