function mealsController(ScreenService, MealsService, GeneralMethods, ZHPromise, MessageService, templateManager) {

    var numberMask = {
        type: "number"
    };
    var currencyMask = {
        type: "currency",
        params: {
            symbol: "",
            showSymbol: false,
            thousands: ".",
            decimal: ",",
            precision: 2
        }
    };
    var nonEditablePlate = "X";
    var scope = this;
    /**
     * Open filter if meals widget isn't filtered
     * @param  {Widget} mealsWidget
     * @todo set calendar max date
     */
    this.onEnter = function(mealsWidget) {
        mealsWidget.activate();
        if (!mealsWidget.filtered) {
            var filterWidget = mealsWidget.container.getWidget("filter");
            ScreenService.openPopup(filterWidget);
        }
    };

    /**
     * Validates and apply filter
     * @param  {Widget} filterWidget
     */
    this.applyFilter = function(filterWidget) {
        if (filterWidget.isValid()) {
            var mealsWidget = filterWidget.container.getWidget("meals");
            var dataSourceFilter = getAndUpdateDefaultFilter(mealsWidget);
            dataSourceFilter.push(MealsService.getMenuDateObjFilter(filterWidget.currentRow.DTCARDAPIO));
            mealsWidget.dataSourceFilter = dataSourceFilter;
            mealsWidget.reload().then(function() {
                ScreenService.closePopup(true);
                mealsWidget.filtered = true;
            }, requestErrorCallback);
        }
    };

    this.onEnterHistoricoTotal = function(widgetHistoricoTotal) {
        widgetHistoricoTotal.reload();
    };

    this.carregaHistorico = function(mealsWidget) {
        let currentRowMeals = mealsWidget.currentRow;
        let widgetHistorico = mealsWidget.container.getWidget("historyGrid");
        widgetHistorico.dataSourceFilter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": currentRowMeals.CDFILIAL
            },
            {
                "name": "CDSERVFILI",
                "operator": "=",
                "value": currentRowMeals.CDSERVFILI
            },
            {
                "name": "DTCARDAPIO",
                "operator": "=",
                "value": currentRowMeals.DTCARDAPIO
            },
            {
                "name": "CDOPERADOR",
                "operator": "=",
                "value": currentRowMeals.CDOPERADOR
            }
		];
        widgetHistorico.reload().then(function() {
            scope.selectMeal(mealsWidget.container.getWidget('tabbed'));
        });
    };

    function getAndUpdateDefaultFilter(mealsWidget) {
        var dataSourceFilter = [];
        if (mealsWidget.defaultFilter) {
            dataSourceFilter = angular.copy(mealsWidget.defaultFilter);
        } else {
            dataSourceFilter = [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": GeneralMethods.user.cdfilial
                },
                {
                    "name": "CDOPERADOR",
                    "operator": "=",
                    "value": GeneralMethods.user.cdoperador
                },
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": GeneralMethods.user.nrorg
                }
            ];
            mealsWidget.defaultFilter = angular.copy(dataSourceFilter);
        }
        return dataSourceFilter;
    }

    this.selectMeal = function(tabbedWidget) {
        tabbedWidget.descriptionOrder = tabbedWidget.parent.descriptionOrder;
        tabbedWidget.container.getWidget("meal").refresh = true;
        tabbedWidget.container.getWidget("plates").refresh = true;
        tabbedWidget.container.getWidget("historyGrid").refresh = true;
        // Swipe aberto
        if (ScreenService.isSwipeOpen()) {
            scope.updateMealWidget(tabbedWidget.container.getWidget("meal"));
            scope.updatePlatesWidget(tabbedWidget.container.getWidget("plates"));
        } else {
            // Abrir swipe
            ScreenService.openSwipe(tabbedWidget);
        }
    };

    this.backToMeals = function(mealsWidget) {
        ScreenService.closeSwipe().then(function() {
            var tabbedWidget = mealsWidget.container.getWidget("tabbed");
            tabbedWidget.lastActiveWidget = mealsWidget;
        });
    };

    this.edit = function(widget) {
        let campoEstadoAprovacao = widget.fields.find(function(campo) {
            return campo.name == "estadoAprovacao";
        });
        if (campoEstadoAprovacao && campoEstadoAprovacao.label == 'PENDENTE') {
            MessageService.showMessageByCode('erroSolicitacaoPendente');
            return;
        } else {
            widget.fields.forEach(function(field) {
                let bloqueio = bloqueioHorarioPreparo(widget);
                if (bloqueio) {
                    MessageService.showMessageByCode('erroBloqTempPrep');
                    return;
                } else {
                    GeneralMethods.edit(widget);
                    if (widget.name === "plates") {
                        var editablePlates = MealsService.editablePlates.map(function(field) {
                            return field.FILTRO + '-' + field.CDPRATO;
                        });
                        if (editablePlates.indexOf(field.name) > -1) {
                            field.readOnly = false;
                        } else {
                            field.readOnly = true;
                        }
                    } else {
                        field.readOnly = false;
                    }
                }
                widget.template = "widget/form.html";
            });
        }
    };

    function bloqueioHorarioPreparo(mealWidget) {
        let currentMeals = mealWidget.parent.parent.currentRow;

        let horasAlteracao = currentMeals.HRALTERACLI;
        if (horasAlteracao) {
            let horas = Number(horasAlteracao.substring(0, 2));
            let minutos = Number(horasAlteracao.substring(2));
            var milisegundosParaSubtrair = (horas * 60 * 60 * 1000) + (minutos * 60 * 1000);

            let dateHoraInicioPreparo = currentMeals.DATA_CMP;
            let parts = dateHoraInicioPreparo.split(/[\/ :]/);

            // Para gerar objeto date o mês é de 0 a 11, por isso subrai 1.
            let month = Number(parts[1]) - 1;

            let dataHoraAtual = new Date();
            let objDateHoraInicioPrep = new Date(parts[2], month, parts[0], parts[3], parts[4], parts[5]);
            let horaLimite = new Date(objDateHoraInicioPrep.getTime() - milisegundosParaSubtrair);

            if (dataHoraAtual >= horaLimite) {
                return true;
            } else {
                return false;
            }
        } else {
            return false;
        }
    }

    this.view = function(widget) {
        widget.fields.forEach(function(field) {
            field.readOnly = true;
        });
        widget.view();
        widget.currentRow.$error = {};
        widget.reload();
        widget.template = "widget/form.html";
    };

    this.updatePlatesWidget = function(platesWidget) {
        if (platesWidget.refresh) {
            var currentMeal = platesWidget.container.getWidget("meals").currentRow;
            var menuDate = moment(currentMeal.DTCARDAPIO, "DD/MM/YYYY H:m:s").format("DD/MM/YYYY");
            platesWidget.fields = [];
            MealsService.getPlates(menuDate, currentMeal.CDSERVFILI).then(function(response) {
                var currentRow = {};
                setPlatesWidgetFields(platesWidget, response, currentRow);
                platesWidget.setCurrentRow(currentRow);
                scope.view(platesWidget);
                platesWidget.refresh = false;
            }, requestErrorCallback);
        }
    };

    function setPlatesWidgetFields(platesWidget, plates, currentRow) {
        var editablePlates = getEditablePlates(plates);
        var nonEditablePlates = getNonEditablePlates(plates);
        MealsService.editablePlates = editablePlates;
        setPlatesFields(platesWidget, editablePlates, currentRow, false);
        setPlatesFields(platesWidget, nonEditablePlates, currentRow, true);
    }

    function getEditablePlates(plates) {
        return plates.filter(function(plate) {
            return plate.FILTRO !== nonEditablePlate;
        });
    }

    function getNonEditablePlates(plates) {
        return plates.filter(function(plate) {
            return plate.FILTRO === nonEditablePlate;
        });
    }

    this.updateMealWidget = function(mealWidget) {
        if (mealWidget.refresh) {
            var currentMeal = mealWidget.container.getWidget("meals").currentRow;
            var menuDate = moment(currentMeal.DTCARDAPIO, "DD/MM/YYYY H:m:s").format("DD/MM/YYYY");
            mealWidget.fields = [];
            scope.view(mealWidget);
            MealsService.getDeliveryPointsAndShifts(menuDate, currentMeal.CDSERVFILI).then(function(response) {
                var currentRow = {};
                setMealWidgetFields(mealWidget, response, currentRow, currentMeal);
                mealWidget.setCurrentRow(currentRow);
                mealWidget.refresh = false;
            }, requestErrorCallback);
        }
    };

    function setMealWidgetFields(mealWidget, obj, currentRow, currentMeal) {
        MealsService.deliveryPoints = obj.deliveryPoints;
        MealsService.shifts = obj.shifts;
        setDefaultField(mealWidget, currentRow, currentMeal);
        setShiftsFields(mealWidget, MealsService.shifts, currentRow);
        setDeliveryPointsFields(mealWidget, MealsService.deliveryPoints, currentRow);
    }

    this.abreObservacao = function(widget) {
        try {
            let shiftfFields = getFieldsByNameLike(widget.fields, "TURNO");
            let deliveryPointsFields = getFieldsByNameLike(widget.fields, "PONTO");
            let rowEdited = widget.currentRow;
            let quantity = parseInt(rowEdited.COMENSAIS);
            let obsWidget = widget.container.getWidget("observacaoWidget");
            validateShifts(shiftfFields, quantity);
            validateDeliveryPoints(deliveryPointsFields, quantity);
            obsWidget.currentRow.OBSERVACAO = " ";
            ScreenService.openPopup(obsWidget);
        } catch (errorCode) {
            MessageService.showMessageByCode(errorCode);
        }
    };

    this.salvarObservacao = function(widget) {
        let observacao = widget.currentRow.OBSERVACAO;
        let mealWidget = widget.container.getWidget("meal");
        scope.saveMeal(mealWidget, observacao);
        ScreenService.closePopup();
    };

    this.defaultFieldsPopUpHistorico = function(widgetHistorico) {
        let popupHistorico = templateManager.container.getWidget("historyDados");
        popupHistorico.currentRow.DTINCLUSAO = widgetHistorico.currentRow.DTINCLUSAO;
        popupHistorico.currentRow.IDSITUACAO = widgetHistorico.currentRow.IDSITUACAO;
        popupHistorico.currentRow.QTCOMENSAIS = widgetHistorico.currentRow.QTCOMENSAIS;
        popupHistorico.currentRow.DSOBSERVACAO = widgetHistorico.currentRow.DSOBSERVACAO;
        popupHistorico.currentRow.DSJUSTIFICATIVA = widgetHistorico.currentRow.DSJUSTIFICATIVA;

        let campoDtinclusao = {
            "name": "DTINCLUSAO",
            "label": "Data",
            "readOnly": true,
            "template": "rui-fields#field/text.html",
            "isVisible": true,
            "dateFormat": "DD/MM/YYYY",
            "displayFormat": "DD/MM/YYYY",
            "defaultValue": "@moment().format('DD/MM/YYYY')@",
            "events": [],
            "class": 6
        };
        GeneralMethods.createAndAppendField(popupHistorico, campoDtinclusao);

        let campoIdsituacao = {
            "name": "IDSITUACAO",
            "label": "Situação",
            "readOnly": true,
            "template": "rui-fields#field/text.html",
            "isVisible": true,
            "showOnForm": true,
            "showOnList": true,
            "showOnView": true,
            "class": 6
        };
        GeneralMethods.createAndAppendField(popupHistorico, campoIdsituacao);

        let campoQtcomensais = {
            "name": "QTCOMENSAIS",
            "label": "Quantidade",
            "readOnly": true,
            "template": "rui-fields#field/text.html",
            "isVisible": true,
            "showOnForm": true,
            "showOnList": true,
            "showOnView": true,
            "class": 12
        };
        GeneralMethods.createAndAppendField(popupHistorico, campoQtcomensais);

        let campoDsobservacao = {
            "name": "DSOBSERVACAO",
            "label": "Observação",
            "readOnly": true,
            "template": "rui-fields#field/text.html",
            "isVisible": true,
            "fieldGroup": "observacao",
            "showOnForm": true,
            "showOnList": true,
            "showOnView": true,
            "class": 12
        };
        GeneralMethods.createAndAppendField(popupHistorico, campoDsobservacao);

        let campoDsjustificativa = {
            "name": "DSJUSTIFICATIVA",
            "label": "Justificativa",
            "readOnly": true,
            "template": "rui-fields#field/text.html",
            "isVisible": true,
            "fieldGroup": "justificativa",
            "showOnForm": true,
            "showOnList": true,
            "showOnView": true,
            "class": 12
        };
        GeneralMethods.createAndAppendField(popupHistorico, campoDsjustificativa);
    };

    this.defaultFieldsPopUpHistoricoTotal = function(widgetHistoricoTotal) {
        let popupHistoricoTotal = templateManager.container.getWidget("historyDadosTotal");
        popupHistoricoTotal.currentRow.DTINCLUSAO = widgetHistoricoTotal.currentRow.DTINCLUSAO;
        popupHistoricoTotal.currentRow.IDSITUACAO = widgetHistoricoTotal.currentRow.IDSITUACAO;
        popupHistoricoTotal.currentRow.QTCOMENSAIS = widgetHistoricoTotal.currentRow.QTCOMENSAIS;
        popupHistoricoTotal.currentRow.DSOBSERVACAO = widgetHistoricoTotal.currentRow.DSOBSERVACAO;
        popupHistoricoTotal.currentRow.DSJUSTIFICATIVA = widgetHistoricoTotal.currentRow.DSJUSTIFICATIVA;

        let campoDtinclusao = {
            "name": "DTINCLUSAO",
            "label": "Data",
            "readOnly": true,
            "template": "rui-fields#field/text.html",
            "isVisible": true,
            "dateFormat": "DD/MM/YYYY",
            "displayFormat": "DD/MM/YYYY",
            "defaultValue": "@moment().format('DD/MM/YYYY')@",
            "events": [],
            "class": 6
        };
        GeneralMethods.createAndAppendField(popupHistoricoTotal, campoDtinclusao);

        let campoIdsituacao = {
            "name": "IDSITUACAO",
            "label": "Situação",
            "readOnly": true,
            "template": "rui-fields#field/text.html",
            "isVisible": true,
            "showOnForm": true,
            "showOnList": true,
            "showOnView": true,
            "class": 6
        };
        GeneralMethods.createAndAppendField(popupHistoricoTotal, campoIdsituacao);

        let campoQtcomensais = {
            "name": "QTCOMENSAIS",
            "label": "Quantidade",
            "readOnly": true,
            "template": "rui-fields#field/text.html",
            "isVisible": true,
            "showOnForm": true,
            "showOnList": true,
            "showOnView": true,
            "class": 12
        };
        GeneralMethods.createAndAppendField(popupHistoricoTotal, campoQtcomensais);

        let campoDsobservacao = {
            "name": "DSOBSERVACAO",
            "label": "Observação",
            "readOnly": true,
            "template": "rui-fields#field/text.html",
            "isVisible": true,
            "fieldGroup": "observacao",
            "showOnForm": true,
            "showOnList": true,
            "showOnView": true,
            "class": 12
        };
        GeneralMethods.createAndAppendField(popupHistoricoTotal, campoDsobservacao);

        let campoDsjustificativa = {
            "name": "DSJUSTIFICATIVA",
            "label": "Justificativa",
            "readOnly": true,
            "template": "rui-fields#field/text.html",
            "isVisible": true,
            "fieldGroup": "justificativa",
            "showOnForm": true,
            "showOnList": true,
            "showOnView": true,
            "class": 12
        };
        GeneralMethods.createAndAppendField(popupHistoricoTotal, campoDsjustificativa);
    };

    this.abreSwipeHistorico = function(widgetHistorico) {
        let popupHistorico = templateManager.container.getWidget("historyDados");
        popupHistorico.fields = [];
        let nrhistoricomymeals = {
            'NRHISTORICOMYMEALS': widgetHistorico.currentRow.NRHISTORICOMYMEALS
        };
        MealsService.getDeliveryPointsAndShiftsHistory(nrhistoricomymeals).then(function(response) {
            scope.defaultFieldsPopUpHistorico(widgetHistorico);
            let turnosRetorno = response.dataset.dadosRetorno.TURNOS;
            let pontosEntregaRetorno = response.dataset.dadosRetorno.PONTO_ENTREGA;
            let comensaisTotalRetorno = response.dataset.dadosRetorno.COMENSAIS[0].QTCOMEALTERADA;
            turnosRetorno.forEach(function(shift) {
                let comensaisTurnos = parseFloat(shift.QTCOMEALTERADA);
                comensaisTurnos = comensaisTurnos >= 0 ? Math.ceil(comensaisTurnos) : 0;
                let propertiesTurnos = {
                    "name": "TURNO" + shift.CDTURNO,
                    "label": shift.NMTURNO,
                    "shifitCode": shift.CDTURNO,
                    "defaultValue": comensaisTurnos,
                    "fieldGroup": "turnos"
                };
                popupHistorico.currentRow[propertiesTurnos.name] = propertiesTurnos.defaultValue;
                GeneralMethods.createAndAppendField(popupHistorico, propertiesTurnos);
            });
            pontosEntregaRetorno.forEach(function(deliveryPoint) {
                var comensais = parseFloat(deliveryPoint.QTCOMEALTERADA);
                comensais = comensais >= 0 ? Math.ceil(comensais) : 0;
                var properties = {
                    "name": "PONTO" + deliveryPoint.CDDESTFILI,
                    "label": deliveryPoint.NMDESTFILI,
                    "deliveryPointCode": deliveryPoint.CDDESTFILI,
                    "defaultValue": comensais,
                    "fieldGroup": "pontosEntrega"
                };
                popupHistorico.currentRow[properties.name] = properties.defaultValue;
                GeneralMethods.createAndAppendField(popupHistorico, properties);
                popupHistorico.currentRow.QTCOMENSAIS = comensaisTotalRetorno;
            });
        }, requestErrorCallback);
        ScreenService.openPopup(popupHistorico);
    };

    this.abreSwipeHistoricoTotal = function(widgetHistoricoTotal) {
        var popupHistoricoTotal = templateManager.container.getWidget("historyDadosTotal");
        popupHistoricoTotal.fields = [];
        let nrhistoricomymeals = {
            'NRHISTORICOMYMEALS': widgetHistoricoTotal.currentRow.NRHISTORICOMYMEALS
        };
        MealsService.getDeliveryPointsAndShiftsHistory(nrhistoricomymeals).then(function(response) {
            scope.defaultFieldsPopUpHistoricoTotal(widgetHistoricoTotal);
            let turnosRetorno = response.dataset.dadosRetorno.TURNOS;
            let pontosEntregaRetorno = response.dataset.dadosRetorno.PONTO_ENTREGA;
            let comensaisTotalRetorno = response.dataset.dadosRetorno.COMENSAIS[0].QTCOMEALTERADA;
            turnosRetorno.forEach(function(shift) {
                let comensaisTurnos = parseFloat(shift.QTCOMEALTERADA);
                comensaisTurnos = comensaisTurnos >= 0 ? Math.ceil(comensaisTurnos) : 0;
                let propertiesTurnos = {
                    "name": "TURNO" + shift.CDTURNO,
                    "label": shift.NMTURNO,
                    "shifitCode": shift.CDTURNO,
                    "defaultValue": comensaisTurnos,
                    "fieldGroup": "turnos"
                };
                popupHistoricoTotal.currentRow[propertiesTurnos.name] = propertiesTurnos.defaultValue;
                GeneralMethods.createAndAppendField(popupHistoricoTotal, propertiesTurnos);
            });
            pontosEntregaRetorno.forEach(function(deliveryPoint) {
                var comensais = parseFloat(deliveryPoint.QTCOMEALTERADA);
                comensais = comensais >= 0 ? Math.ceil(comensais) : 0;
                var properties = {
                    "name": "PONTO" + deliveryPoint.CDDESTFILI,
                    "label": deliveryPoint.NMDESTFILI,
                    "deliveryPointCode": deliveryPoint.CDDESTFILI,
                    "defaultValue": comensais,
                    "fieldGroup": "pontosEntrega"
                };
                popupHistoricoTotal.currentRow[properties.name] = properties.defaultValue;
                GeneralMethods.createAndAppendField(popupHistoricoTotal, properties);
                popupHistoricoTotal.currentRow.QTCOMENSAIS = comensaisTotalRetorno;
            });
        }, requestErrorCallback);
        ScreenService.openPopup(popupHistoricoTotal);
    };

    this.mostraJustificativaReprovado = function() {
        let widgetHistorico = templateManager.container.getWidget("historyGrid");
        let dadosHistorico = widgetHistorico.dataSource.data;
        let labelEstadoAprovacao = dadosHistorico[0].IDSITUACAO;
        let textoJustificativa = dadosHistorico[0].DSJUSTIFICATIVA;

        if (labelEstadoAprovacao == 'REPROVADO') {
            ScreenService.showMessage(textoJustificativa);
        }
    };

    function setDefaultField(mealWidget, currentRow, currentMeal) {
        var comensais = parseFloat(currentMeal.COMENSAIS);
        var widgetHistorico = mealWidget.container.getWidget("historyGrid");
        var dadosHistorico = widgetHistorico.dataSource.data;

        if (dadosHistorico.length > 0) {
            var labelEstadoAprovacao = dadosHistorico[0].IDSITUACAO;
            var botaoStatusAprovacao = {
                "name": "estadoAprovacao",
                "label": labelEstadoAprovacao,
                "class": "2 estadoAprovacao-btn",
                "isVisible": true,
                "template": "rui-fields#field/button.html",
                "events": [
                    {
                        "name": "ActionEvent",
                        "code": "MealsController.mostraJustificativaReprovado();"
                    }
                ]
            };
            if (labelEstadoAprovacao == 'PENDENTE') {
                mealWidget.actions.forEach(function(obj) {
                    if (obj.name === "edit") {
                        obj.readOnly = true;
                    }
                });
            }

            if (labelEstadoAprovacao == 'REPROVADO') {
                GeneralMethods.createAndAppendFieldButton(mealWidget, botaoStatusAprovacao);
            } else {
                GeneralMethods.createAndAppendFieldNovalidation(mealWidget, botaoStatusAprovacao);
            }

        }

        comensais = comensais >= 0 ? Math.ceil(comensais) : 0;
        var properties = {
            "name": "COMENSAIS",
            "label": "Quantity",
            "defaultValue": comensais,
            "mask": numberMask,
            "type": "tel",
            "validations": {
                "required": true
            }
        };
        currentRow[properties.name] = properties.defaultValue;
        GeneralMethods.createAndAppendField(mealWidget, properties);
    }

    function setShiftsFields(mealWidget, shifts, currentRow) {
        shifts.forEach(function(shift) {
            var comensais = parseFloat(shift.COMENSAIS);
            comensais = comensais >= 0 ? Math.ceil(comensais) : 0;
            var properties = {
                "name": "TURNO" + shift.CDTURNO,
                "label": shift.NMTURNO,
                "shifitCode": shift.CDTURNO,
                "defaultValue": comensais,
                "fieldGroup": "shifts",
                "type": "tel",
                "mask": numberMask
            };
            currentRow[properties.name] = properties.defaultValue;
            GeneralMethods.createAndAppendField(mealWidget, properties);
        });
        var isVisible = shifts.length ? true : false;
        toggleFieldGroupVisibility(mealWidget, "shifts", isVisible);
    }

    function setDeliveryPointsFields(mealWidget, deliveryPoints, currentRow) {
        deliveryPoints.forEach(function(deliveryPoint) {
            var comensais = parseFloat(deliveryPoint.COMENSAIS);
            comensais = comensais >= 0 ? Math.ceil(comensais) : 0;
            var properties = {
                "name": "PONTO" + deliveryPoint.CDDESTFILI,
                "label": deliveryPoint.NMDESTFILI,
                "deliveryPointCode": deliveryPoint.CDDESTFILI,
                "defaultValue": comensais,
                "fieldGroup": "deliveryPoints",
                "type": "tel",
                "mask": numberMask
            };
            currentRow[properties.name] = properties.defaultValue;
            GeneralMethods.createAndAppendField(mealWidget, properties);
        });
        var isVisible = deliveryPoints.length ? true : false;
        toggleFieldGroupVisibility(mealWidget, "deliveryPoints", isVisible);
    }

    function setPlatesFields(platesWidget, plates, currentRow) {
        plates.forEach(function(plate) {
            var properties = {
                "name": plate.CDTIPOPRAT + "-" + plate.CDPRATO,
                "label": plate.NMPRATO,
                "defaultValue": plate.COMENSAIS,
                "readOnly": true,
                "mask": currencyMask
            };
            currentRow[properties.name] = properties.defaultValue;
            GeneralMethods.createAndAppendField(platesWidget, properties);
        });
    }

    function toggleFieldGroupVisibility(widget, name, isVisible) {
        for (var index in widget.fieldGroups) {
            var fieldGroup = widget.fieldGroups[index];
            if (fieldGroup.name === name) {
                fieldGroup.isVisible = isVisible;
            }
        }
    }

    this.saveMeal = function(mealWidget, observacao) {
        try {
            if (mealWidget.isValid()) {
                var rowEdited = mealWidget.currentRow;
                var quantity = parseInt(rowEdited.COMENSAIS);
                var currentMeal = mealWidget.parent.parent.currentRow;
                currentMeal.OBSERVACAO = observacao;
                var emailMessages;
                emailMessages = ['emailSubject', 'emailFromName', 'emailAcima',
                'emailAbaixo', 'emailDescription', 'emailFooter',
                'emailPendente', 'emailAprovada', 'emailDataCardapio',
                'emailOperador', 'emailUnidade', 'emailServico',
                'emailQtOri', 'emailQtAlt', 'emailSituacao'];
                var emailMessagesObj;
                emailMessagesObj = {};
                var getMessage;
                getMessage = function(idx) {
                    return MessageService.getMessageByCode(idx).then(function(message) {
                        emailMessagesObj[idx] = message;
                    });
                };
                var emailMessagesPromises;
                emailMessagesPromises = emailMessages.map(getMessage);
                if (!quantity || quantity === 0) {
                    MessageService.confirmMessageByCode('desejaZerarComensais').then(
                        function() {
                            window.Promise.all(emailMessagesPromises).then(function() {
                                MealsService.save(currentMeal, rowEdited, emailMessagesObj).then(function() {
                                    scope.view(mealWidget);
                                    mealWidget.container.getWidget("plates").refresh = true;
                                    mealWidget.container.getWidget("historyGrid").reload().then(function() {
                                        mealWidget.fields = [];
                                        mealWidget.refresh = true;
                                        scope.updateMealWidget(mealWidget);
                                    });
                                }).catch(requestErrorCallback);
                            });
                        },
                        function() {
                            return;
                        }
                    );
                } else {
                    window.Promise.all(emailMessagesPromises).then(function() {
                        MealsService.save(currentMeal, rowEdited, emailMessagesObj).then(function() {
                            scope.view(mealWidget);
                            mealWidget.container.getWidget("plates").refresh = true;
                            mealWidget.container.getWidget("historyGrid").reload().then(function() {
                                mealWidget.fields = [];
                                mealWidget.refresh = true;
                                scope.updateMealWidget(mealWidget);
                            });
                        }).catch(requestErrorCallback);
                    });
                }
            }
        } catch (errorCode) {
            MessageService.showMessageByCode(errorCode);
        }
    };

    function getFieldsByNameLike(fields, name) {
        return fields.filter(function(field) {
            return field.name.indexOf(name) !== -1;
        });
    }

    function validateShifts(shiftfFields, quantity) {
        if (shiftfFields.length) {
            var shiftsSum = getSum(shiftfFields);
            if (shiftsSum !== quantity) {
                throw "shiftNotEqualQuantity";
            }
        }
    }

    function validateDeliveryPoints(deliveryPointsFields, quantity) {
        if (deliveryPointsFields.length) {
            var deliveryPointsSum = getSum(deliveryPointsFields);
            if (deliveryPointsSum !== quantity) {
                throw "deliveryPointsNotEqualQuantity";
            }
        }
    }

    function getSum(fields) {
        return fields.map(function(field) {
            return parseInt(field.value());
        }).reduce(function(lastValue, currentValue) {
            return lastValue + currentValue;
        }, 0);
    }

    this.savePlates = function(platesWidget) {
        try {
            if (platesWidget.isValid()) {
                var changedPlates = getChangedPlates(platesWidget.currentRow, platesWidget.backupRow);
                if (Object.keys(changedPlates).length) {
                    var currentMeal = platesWidget.parent.parent.currentRow;
                    MealsService.savePlates(changedPlates, currentMeal).then(function() {
                        platesWidget.refresh = true;
                        scope.updatePlatesWidget(platesWidget);
                    }).catch(requestErrorCallback);
                } else {
                    GeneralMethods.cancelEdit(platesWidget);
                }
            }
        } catch (errorCode) {
            MessageService.showMessageByCode(errorCode);
        }
    };

    function getChangedPlates(currentRow, backupRow) {
        var changedPlates = {};
        for (var attr in backupRow) {
            if (attr !== "$error" && currentRow[attr] !== backupRow[attr]) {
                changedPlates[attr] = currentRow[attr];
            }
        }
        return changedPlates;
    }

    this.onCloseSwipe = function(tabbedWidget) {
        var mealWidget = tabbedWidget.container.getWidget("meal");
        var platesWidget = tabbedWidget.container.getWidget("plates");
        var mealsWidget = tabbedWidget.container.getWidget("meals");
        GeneralMethods.cancelEdit(mealWidget);
        GeneralMethods.cancelEdit(platesWidget);
        tabbedWidget.lastActiveWidget = mealsWidget;
    };

    this.mealsOnBack = function(widget) {
        if (widget.mode === "edit") {
            GeneralMethods.cancelEdit(widget);
        } else {
            this.backToMeals(widget);
        }
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('MealsController', mealsController);
});
